# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from babel.numbers import LC_NUMERIC
from babel.numbers import format_currency as babel_format_currency


def format_money(money, format=None, locale=LC_NUMERIC, currency_digits=True,
                 format_type='standard', decimal_quantization=True):
    """
    See https://babel.pocoo.org/en/latest/api/numbers.html
    """
    return babel_format_currency(money.amount, money.currency.code,
                                 format=format, locale=locale, currency_digits=currency_digits,
                                 format_type=format_type, decimal_quantization=decimal_quantization)



# -*- encoding: utf-8 -*-

from __future__ import division, unicode_literals

import warnings
from copy import deepcopy
from decimal import Decimal

import pytest  # Works with less code, more consistency than unittest.
from babel.core import get_global

from moneyed.classes import (CURRENCIES, DEFAULT_CURRENCY, PYTHON2, USD,
                             Currency, Money, MoneyComparisonError,
                             force_decimal, get_currencies_of_country,
                             get_currency, list_all_currencies)
from moneyed.localization import format_money

if not PYTHON2:
    unicode = str  # Avoid UndefinedName flake8 errors in Python3


class CustomDecimal(Decimal):
    """Test class to ensure Decimal.__str__ is not
    used in calculations.
    """
    def __str__(self):
        return 'error'


class TestCurrency:

    def setup_method(self, method):
        self.default_curr_code = 'XYZ'
        self.default_curr = CURRENCIES[self.default_curr_code]

    def test_init(self):
        US_dollars = Currency(
            code='USD',
            numeric='840',
            sub_unit=100,
            name='United States Dollar',  # NB deliberately not official name
            countries=['UNITED STATES'])
        assert US_dollars.code == 'USD'
        assert US_dollars.countries == ['UNITED STATES']
        assert US_dollars.name == 'United States Dollar'
        assert US_dollars.numeric == '840'
        assert US_dollars.sub_unit == 100

    def test_name(self):
        assert USD.name == "US Dollar"

    def test_countries(self):
        assert CURRENCIES['AED'].countries == ['UNITED ARAB EMIRATES']

    def test_get_name(self):
        assert USD.get_name('es') == 'dólar estadounidense'
        assert USD.get_name('en_GB', count=10) == 'US dollars'

    def test_repr(self):
        assert str(self.default_curr) == self.default_curr_code

    def test_hash(self):
        assert self.default_curr in set([self.default_curr])

    def test_compare(self):
        other = deepcopy(self.default_curr)
        # equality
        assert self.default_curr == CURRENCIES['XYZ']
        assert self.default_curr == other
        # non-equality
        other.code = 'USD'
        assert self.default_curr != other
        assert self.default_curr != CURRENCIES['USD']

    def test_fetching_currency_by_iso_code(self):
        assert get_currency('USD') == USD
        assert get_currency(iso='840') == USD
        assert get_currency(iso=840) == USD

    def test_get_currencies_of_country(self):
        assert get_currencies_of_country("IN")[0] == Currency('INR')
        assert get_currencies_of_country("iN")[0] == Currency('INR')
        assert get_currencies_of_country("BT") == [Currency('BTN'), Currency('INR')]
        assert get_currencies_of_country("XX") == []


class TestMoney:

    def setup_method(self, method):
        self.one_million_decimal = Decimal('1000000')
        self.USD = CURRENCIES['USD']
        self.one_million_bucks = Money(amount=self.one_million_decimal,
                                       currency=self.USD)

    def test_init(self):
        one_million_dollars = Money(amount=self.one_million_decimal,
                                    currency=self.USD)
        assert one_million_dollars.amount == self.one_million_decimal
        assert one_million_dollars.currency == self.USD

    def test_init_string_currency_code(self):
        one_million_dollars = Money(amount=self.one_million_decimal,
                                    currency='usd')
        assert one_million_dollars.amount == self.one_million_decimal
        assert one_million_dollars.currency == self.USD

    def test_init_default_currency(self):
        one_million = self.one_million_decimal
        one_million_dollars = Money(amount=one_million)  # No currency given!
        assert one_million_dollars.amount == one_million
        assert one_million_dollars.currency == DEFAULT_CURRENCY

    def test_init_float(self):
        one_million_dollars = Money(amount=1000000.0)
        assert one_million_dollars.amount == self.one_million_decimal

    def test_repr(self):
        assert repr(self.one_million_bucks) == "Money('1000000', 'USD')"
        assert repr(Money(Decimal('2.000'), 'PLN')) == "Money('2.000', 'PLN')"
        m_1 = Money(Decimal('2.00'), 'PLN')
        m_2 = Money(Decimal('2.01'), 'PLN')
        assert repr(m_1) != repr(m_2)

    def test_eval_from_repr(self):
        m = Money('1000', 'USD')
        assert m == eval(repr(m))

    def test_str(self):
        if PYTHON2:
            # Bytestring uses a simpler fallback to avoid unicode chars
            assert str(self.one_million_bucks) == 'USD1,000,000'

        # Conversion to text use default locale, so results vary
        # depending on system setup. Just assert that we don't crash.
        if PYTHON2:
            assert isinstance(unicode(self.one_million_bucks), unicode)
        else:
            assert isinstance(str(self.one_million_bucks), str)

    def test_hash(self):
        assert self.one_million_bucks in set([self.one_million_bucks])

    def test_format_money(self):
        # Two decimal places by default
        assert format_money(self.one_million_bucks) == 'US$1,000,000.00'
        # No decimal point without fractional part
        assert format_money(self.one_million_bucks, decimal_places=0) == 'US$1,000,000'
        # Locale format not included, should fallback to DEFAULT
        assert format_money(self.one_million_bucks, locale='es_ES') == 'US$1,000,000.00'
        # locale == pl_PL
        one_million_pln = Money('1000000', 'PLN')
        # Two decimal places by default
        assert format_money(one_million_pln, locale='pl_PL') == '1 000 000,00 zł'
        assert format_money(self.one_million_bucks, locale='pl_PL') == 'US$1 000 000,00'
        # No decimal point without fractional part
        assert format_money(one_million_pln, locale='pl_PL',
                            decimal_places=0) == '1 000 000 zł'

    def test_format_money_fr(self):
        # locale == fr_FR
        one_million_eur = Money('1000000', 'EUR')
        one_million_cad = Money('1000000', 'CAD')
        assert format_money(one_million_eur, locale='fr_FR') == '1 000 000,00 €'
        assert format_money(self.one_million_bucks, locale='fr_FR') == '1 000 000,00 $ US'
        assert format_money(one_million_cad, locale='fr_FR') == '1 000 000,00 $ CA'
        # No decimal point without fractional part
        assert format_money(one_million_eur, locale='fr_FR',
                            decimal_places=0) == '1 000 000 €'
        # locale == fr_CA
        assert format_money(one_million_cad, locale='fr_CA') == '1 000 000,00 $'
        assert format_money(self.one_million_bucks, locale='fr_CA') == '1 000 000,00 $ US'
        assert format_money(one_million_eur, locale='fr_CA') == '1 000 000,00 €'

    def test_add(self):
        assert (self.one_million_bucks + self.one_million_bucks ==
                Money(amount='2000000', currency=self.USD))

    def test_add_non_money(self):
        with pytest.raises(TypeError):
            Money(1000) + 123

    def test_sub(self):
        zeroed_test = self.one_million_bucks - self.one_million_bucks
        assert zeroed_test == Money(amount=0, currency=self.USD)

    def test_sub_non_money(self):
        with pytest.raises(TypeError):
            Money(1000) - 123

    def test_rsub_non_money(self):
        assert 0 - Money(1, currency=self.USD) == Money(-1, currency=self.USD)
        with pytest.raises(TypeError):
            assert 1 - Money(3, currency=self.USD) == Money(-2, currency=self.USD)

    def test_mul(self):
        x = Money(amount=111.33, currency=self.USD)
        assert 3 * x == Money(333.99, currency=self.USD)
        assert Money(333.99, currency=self.USD) == 3 * x

    def test_mul_float_warning(self):
        # This should be changed to TypeError exception after deprecation period is over.
        with warnings.catch_warnings(record=True) as warning_list:
            warnings.simplefilter("always")
            Money(amount="10") * 1.2
            assert "Multiplying Money instances with floats is deprecated" in [w.message.args[0] for w in warning_list]

        with warnings.catch_warnings(record=True) as warning_list:
            warnings.simplefilter("always")
            1.2 * Money(amount="10")
            assert "Multiplying Money instances with floats is deprecated" in [w.message.args[0] for w in warning_list]

    def test_mul_bad(self):
        with pytest.raises(TypeError):
            self.one_million_bucks * self.one_million_bucks

    def test_div(self):
        x = Money(amount=50, currency=self.USD)
        y = Money(amount=2, currency=self.USD)
        assert x / y == Decimal(25)

    def test_div_mismatched_currencies(self):
        x = Money(amount=50, currency=self.USD)
        y = Money(amount=2, currency=CURRENCIES['CAD'])
        with pytest.raises(TypeError):
            assert x / y == Money(amount=25, currency=self.USD)

    def test_div_by_non_Money(self):
        x = Money(amount=50, currency=self.USD)
        y = 2
        assert x / y == Money(amount=25, currency=self.USD)

    def test_rdiv_by_non_Money(self):
        x = 2
        y = Money(amount=50, currency=self.USD)
        with pytest.raises(TypeError):
            assert x / y == Money(amount=25, currency=self.USD)

    def test_div_float_warning(self):
        # This should be changed to TypeError exception after deprecation period is over.
        with warnings.catch_warnings(record=True) as warning_list:
            warnings.simplefilter("always")
            Money(amount="10") / 1.2
            assert "Dividing Money instances by floats is deprecated" in [w.message.args[0] for w in warning_list]

    def test_rmod(self):
        assert 1 % self.one_million_bucks == Money(amount=10000,
                                                   currency=self.USD)

    def test_rmod_bad(self):
        with pytest.raises(TypeError):
            assert (self.one_million_bucks % self.one_million_bucks == 1)

    def test_rmod_float_warning(self):
        # This should be changed to TypeError exception after deprecation period is over.
        with warnings.catch_warnings(record=True) as warning_list:
            warnings.simplefilter("always")
            2.0 % Money(amount="10")
            assert ("Calculating percentages of Money instances using floats is deprecated"
                    in [w.message.args[0] for w in warning_list])

    def test_convert_to_default(self):
        # Currency conversions are not implemented as of 2/2011; when
        # they are working, then convert_to_default and convert_to
        # will need to be tested.
        pass

    # Note: no tests for __eq__ as it's quite thoroughly covered in
    # the assert comparisons throughout these tests.

    def test_ne(self):
        x = Money(amount=1, currency=self.USD)
        assert self.one_million_bucks != x

    def test_equality_to_other_types(self):
        x = Money(amount=0, currency=self.USD)
        assert x != None  # NOQA
        assert x != {}

    def test_not_equal_to_decimal_types(self):
        assert self.one_million_bucks != self.one_million_decimal

    def test_lt(self):
        x = Money(amount=1, currency=self.USD)
        assert x < self.one_million_bucks

    def test_lt_mistyped(self):
        x = 1.0
        with pytest.raises(MoneyComparisonError):
            assert x < self.one_million_bucks

    def test_gt(self):
        x = Money(amount=1, currency=self.USD)
        assert self.one_million_bucks > x

    def test_gt_mistyped(self):
        x = 1.0
        with pytest.raises(MoneyComparisonError):
            assert self.one_million_bucks > x

    def test_abs(self):
        abs_money = Money(amount=1, currency=self.USD)
        x = Money(amount=-1, currency=self.USD)
        assert abs(x) == abs_money
        y = Money(amount=1, currency=self.USD)
        assert abs(y) == abs_money

    def test_sum(self):
        assert (sum([Money(amount=1, currency=self.USD),
                     Money(amount=2, currency=self.USD)]) ==
                Money(amount=3, currency=self.USD))

    def test_round(self):
        x = Money(amount='1234.33569', currency=self.USD)
        assert x.round(-4) == Money(amount='0', currency=self.USD)
        assert x.round(-3) == Money(amount='1000', currency=self.USD)
        assert x.round(-2) == Money(amount='1200', currency=self.USD)
        assert x.round(-1) == Money(amount='1230', currency=self.USD)
        assert x.round(0) == Money(amount='1234', currency=self.USD)
        assert x.round(None) == Money(amount='1234', currency=self.USD)
        assert x.round(1) == Money(amount='1234.3', currency=self.USD)
        assert x.round(2) == Money(amount='1234.34', currency=self.USD)
        assert x.round(3) == Money(amount='1234.336', currency=self.USD)
        assert x.round(4) == Money(amount='1234.3357', currency=self.USD)

    def test_round_context_override(self):
        import decimal

        x = Money(amount='2.5', currency=self.USD)
        assert x.round(0) == Money(amount=2, currency=self.USD)
        x = Money(amount='3.5', currency=self.USD)
        assert x.round(0) == Money(amount=4, currency=self.USD)

        with decimal.localcontext() as ctx:
            ctx.rounding = decimal.ROUND_HALF_UP
            x = Money(amount='2.5', currency=self.USD)
            assert x.round(0) == Money(amount=3, currency=self.USD)
            x = Money(amount='3.5', currency=self.USD)
            assert x.round(0) == Money(amount=4, currency=self.USD)

    def test_get_sub_unit(self):
        m = Money(amount=123, currency=self.USD)
        assert m.get_amount_in_sub_unit() == 12300

    def test_arithmetic_operations_return_real_subclass_instance(self):
        """
        Arithmetic operations on a subclass instance should return instances in the same subclass
        type.
        """

        extended_money = ExtendedMoney(amount=2, currency=self.USD)

        operated_money = +extended_money
        assert type(extended_money) == type(operated_money)
        operated_money = -extended_money
        assert type(extended_money) == type(operated_money)
        operated_money = ExtendedMoney(amount=1, currency=self.USD) + ExtendedMoney(amount=1, currency=self.USD)
        assert type(extended_money) == type(operated_money)
        operated_money = ExtendedMoney(amount=3, currency=self.USD) - Money(amount=1, currency=self.USD)
        assert type(extended_money) == type(operated_money)
        operated_money = (1 * extended_money)
        assert type(extended_money) == type(operated_money)
        operated_money = (extended_money / 1)
        assert type(extended_money) == type(operated_money)
        operated_money = abs(ExtendedMoney(amount=-2, currency=self.USD))
        assert type(extended_money) == type(operated_money)
        operated_money = (50 % ExtendedMoney(amount=4, currency=self.USD))
        assert type(extended_money) == type(operated_money)

    def test_can_call_subclass_method_after_arithmetic_operations(self):
        """
        Calls to `ExtendedMoney::do_my_behaviour` method throws
        AttributeError: 'Money' object has no attribute 'do_my_behaviour'
        if multiplication operator doesn't return subclass instance.
        """

        extended_money = ExtendedMoney(amount=2, currency=self.USD)
        # no problem
        extended_money.do_my_behaviour()
        # throws error if `__mul__` doesn't return subclass instance
        (1 * extended_money).do_my_behaviour()

    def test_bool(self):
        assert bool(Money(amount=1, currency=self.USD))
        assert not bool(Money(amount=0, currency=self.USD))

    def test_force_decimal(self):
        assert force_decimal('53.55') == Decimal('53.55')
        assert force_decimal(53) == Decimal('53')
        assert force_decimal(Decimal('53.55')) == Decimal('53.55')

    def test_decimal_doesnt_use_str_when_multiplying(self):
        m = Money('531', 'GBP')
        a = CustomDecimal('53.313')
        result = m * a
        assert result == Money('28309.203', 'GBP')

    def test_decimal_doesnt_use_str_when_dividing(self):
        m = Money('15.60', 'GBP')
        a = CustomDecimal('3.2')
        result = m / a
        assert result == Money('4.875', 'GBP')


class ExtendedMoney(Money):

    def do_my_behaviour(self):
        pass


def test_all_babel_currencies():
    missing = sorted(list(set(get_global('all_currencies').keys()) - set(CURRENCIES.keys())))
    assert missing == [], \
        'The following currencies defined in Babel are missing: ' + ', '.join(missing)


def test_list_all_currencies():
    all_currencies = list_all_currencies()
    assert len(all_currencies) > 100
    assert [c.code for c in all_currencies[0:3]] == ['ADP', 'AED', 'AFA']
    assert all(isinstance(c, Currency) for c in all_currencies)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from moneyed import Money
from moneyed.l10n import format_money

one_million_bucks = Money('1000000', 'USD')
one_million_eur = Money('1000000', 'EUR')


def test_format_money():
    assert format_money(one_million_bucks, locale='en_US') == '$1,000,000.00'
    assert format_money(one_million_eur, locale='en_US') == '€1,000,000.00'


# Test a few things are being passed on. But don't test everything, it is tested in Babel


def test_format_money_decimal_quantization():
    assert format_money(Money('2.0123', 'USD'), locale='en_US', decimal_quantization=False) == '$2.0123'



class cached_property(object):
    """
    Decorator that creates converts a method with a single
    self argument into a property cached on the instance.
    """
    def __init__(self, func):
        self.func = func
        self.__doc__ = getattr(func, '__doc__')

    def __get__(self, instance, type):
        res = instance.__dict__[self.func.__name__] = self.func(instance)
        return res



# -*- coding: utf-8 -*-
#
# IMPORTANT:
# Please see https://github.com/py-moneyed/py-moneyed/issues/22#issuecomment-447059971 before making changes here.

from __future__ import unicode_literals

import warnings
from decimal import ROUND_HALF_EVEN, Decimal

import moneyed

DEFAULT = "DEFAULT"


warnings.warn('This module and all its contents is deprecated in favour of new moneyed.l10n.format_money.',
              DeprecationWarning)


class CurrencyFormatter(object):

    sign_definitions = {}
    formatting_definitions = {}

    def add_sign_definition(self, locale, currency, prefix='', suffix=''):
        locale = locale.upper()
        currency_code = currency.code.upper()
        if locale not in self.sign_definitions:
            self.sign_definitions[locale] = {}
        self.sign_definitions[locale][currency_code] = (prefix, suffix)

    def add_formatting_definition(self, locale, group_size,
                                  group_separator, decimal_point,
                                  positive_sign, trailing_positive_sign,
                                  negative_sign, trailing_negative_sign,
                                  rounding_method):
        locale = locale.upper()
        self.formatting_definitions[locale] = {
            'group_size': group_size,
            'group_separator': group_separator,
            'decimal_point': decimal_point,
            'positive_sign': positive_sign,
            'trailing_positive_sign': trailing_positive_sign,
            'negative_sign': negative_sign,
            'trailing_negative_sign': trailing_negative_sign,
            'rounding_method': rounding_method}

    def get_sign_definition(self, currency_code, locale):
        currency_code = currency_code.upper()

        if locale.upper() not in self.sign_definitions:
            locale = DEFAULT

        local_set = self.sign_definitions.get(locale.upper())

        if currency_code in local_set:
            return local_set.get(currency_code)
        else:
            ret = self.sign_definitions.get(DEFAULT).get(currency_code)
            return ret if ret is not None else ('', " %s" % currency_code)

    def get_formatting_definition(self, locale):
        if locale.upper() not in self.formatting_definitions:
            locale = DEFAULT
        return self.formatting_definitions.get(locale.upper())

    def format(self, money, include_symbol=True, locale=DEFAULT,
               decimal_places=None, rounding_method=None):
        locale = locale.upper()
        code = money.currency.code.upper()
        prefix, suffix = self.get_sign_definition(code, locale)
        formatting = self.get_formatting_definition(locale)

        if rounding_method is None:
            rounding_method = formatting['rounding_method']

        if decimal_places is None:
            # TODO: Use individual defaults for each currency
            decimal_places = 2

        q = Decimal(10) ** -decimal_places  # 2 places --> '0.01'
        quantized = money.amount.quantize(q, rounding_method)
        negative, digits, e = quantized.as_tuple()

        result = []

        digits = list(map(str, digits))

        build, next = result.append, digits.pop

        # Trailing sign
        if negative:
            build(formatting['trailing_negative_sign'])
        else:
            build(formatting['trailing_positive_sign'])

        # Suffix
        if include_symbol:
            build(suffix)

        # Decimals
        for i in range(decimal_places):
            build(next() if digits else '0')

        # Decimal points
        if decimal_places:
            build(formatting['decimal_point'])

        # Grouped number
        if not digits:
            build('0')
        else:
            i = 0
            while digits:
                build(next())
                i += 1
                if i == formatting['group_size'] and digits:
                    i = 0
                    build(formatting['group_separator'])

        # Prefix
        if include_symbol:
            build(prefix)

        # Sign
        if negative:
            build(formatting['negative_sign'])
        else:
            build(formatting['positive_sign'])

        return ''.join(reversed(result))


_FORMATTER = CurrencyFormatter()

format_money = _FORMATTER.format

_sign = _FORMATTER.add_sign_definition
_format = _FORMATTER.add_formatting_definition

# FORMATTING RULES

_format(DEFAULT, group_size=3, group_separator=",", decimal_point=".",
        positive_sign="", trailing_positive_sign="",
        negative_sign="-", trailing_negative_sign="",
        rounding_method=ROUND_HALF_EVEN)

_format("en_US", group_size=3, group_separator=",", decimal_point=".",
        positive_sign="", trailing_positive_sign="",
        negative_sign="-", trailing_negative_sign="",
        rounding_method=ROUND_HALF_EVEN)

_format("de_DE", group_size=3, group_separator=" ", decimal_point=",",
        positive_sign="", trailing_positive_sign="",
        negative_sign="-", trailing_negative_sign="",
        rounding_method=ROUND_HALF_EVEN)

_format("de_AT", group_size=3, group_separator=" ", decimal_point=",",
        positive_sign="", trailing_positive_sign="",
        negative_sign="-", trailing_negative_sign="",
        rounding_method=ROUND_HALF_EVEN)

_format("de_CH", group_size=3, group_separator=" ", decimal_point=".",
        positive_sign="", trailing_positive_sign="",
        negative_sign="-", trailing_negative_sign="",
        rounding_method=ROUND_HALF_EVEN)

_format("fr_FR", group_size=3, group_separator=" ", decimal_point=",",
        positive_sign="", trailing_positive_sign="",
        negative_sign="-", trailing_negative_sign="",
        rounding_method=ROUND_HALF_EVEN)

_format("fr_CA", group_size=3, group_separator=" ", decimal_point=",",
        positive_sign="", trailing_positive_sign="",
        negative_sign="-", trailing_negative_sign="",
        rounding_method=ROUND_HALF_EVEN)

_format("sv_SE", group_size=3, group_separator=" ", decimal_point=",",
        positive_sign="", trailing_positive_sign="",
        negative_sign="-", trailing_negative_sign="",
        rounding_method=ROUND_HALF_EVEN)

_format("pl_PL", group_size=3, group_separator=" ", decimal_point=",",
        positive_sign="", trailing_positive_sign="",
        negative_sign="-", trailing_negative_sign="",
        rounding_method=ROUND_HALF_EVEN)

_format("en_GB", group_size=3, group_separator=",", decimal_point=".",
        positive_sign="", trailing_positive_sign="",
        negative_sign="-", trailing_negative_sign="",
        rounding_method=ROUND_HALF_EVEN)

_format('es_BO', group_size=3, group_separator=".", decimal_point=",",
        positive_sign="",  trailing_positive_sign="",
        negative_sign="-", trailing_negative_sign="",
        rounding_method=ROUND_HALF_EVEN)

_format("nl_NL", group_size=3, group_separator=".", decimal_point=",",
        positive_sign="", trailing_positive_sign="",
        negative_sign="-", trailing_negative_sign="",
        rounding_method=ROUND_HALF_EVEN)

_format("nb_NO", group_size=3, group_separator=" ", decimal_point=",",
        positive_sign="", trailing_positive_sign="",
        negative_sign="-", trailing_negative_sign="",
        rounding_method=ROUND_HALF_EVEN)

_format("nn_NO", group_size=3, group_separator=" ", decimal_point=",",
        positive_sign="", trailing_positive_sign="",
        negative_sign="-", trailing_negative_sign="",
        rounding_method=ROUND_HALF_EVEN)

# CURRENCY SIGNS
# Default currency signs. These can be overridden for locales where
# foreign or local currency signs for one reason or another differ
# from the norm.

# READ ME FIRST:
#  https://github.com/py-moneyed/py-moneyed/issues/22#issuecomment-447059971

_sign(DEFAULT, moneyed.AED, prefix='د.إ')
_sign(DEFAULT, moneyed.AFN, suffix='؋')
_sign(DEFAULT, moneyed.ALL, prefix='L')
_sign(DEFAULT, moneyed.AMD, prefix='Դ')
_sign(DEFAULT, moneyed.ANG, prefix='ƒ')
_sign(DEFAULT, moneyed.AOA, prefix='Kz')
_sign(DEFAULT, moneyed.ARS, prefix='ARS$')
_sign(DEFAULT, moneyed.AUD, prefix='A$')
_sign(DEFAULT, moneyed.AWG, prefix='ƒ')
_sign(DEFAULT, moneyed.BAM, prefix='КМ')
_sign(DEFAULT, moneyed.BBD, prefix='Bds$')
_sign(DEFAULT, moneyed.BDT, prefix='৳')
_sign(DEFAULT, moneyed.BGN, prefix='лв')
_sign(DEFAULT, moneyed.BHD, prefix='.د.ب')
_sign(DEFAULT, moneyed.BIF, prefix='FBu')
_sign(DEFAULT, moneyed.BMD, prefix='BD$')
_sign(DEFAULT, moneyed.BND, prefix='B$')
_sign(DEFAULT, moneyed.BOB, prefix='Bs.')
_sign(DEFAULT, moneyed.BOV, prefix='Bs.')
_sign(DEFAULT, moneyed.BRL, prefix='R$')
_sign(DEFAULT, moneyed.BSD, prefix='B$')
_sign(DEFAULT, moneyed.BTN, prefix='Nu.')
_sign(DEFAULT, moneyed.BWP, prefix='P')
_sign(DEFAULT, moneyed.BYN, prefix='Br')
_sign(DEFAULT, moneyed.BYR, prefix='Br')
_sign(DEFAULT, moneyed.BZD, prefix='BZ$')
_sign(DEFAULT, moneyed.CAD, prefix='C$')
_sign(DEFAULT, moneyed.CDF, prefix='C₣')
_sign(DEFAULT, moneyed.CHE, prefix='CHE')
_sign(DEFAULT, moneyed.CHF, prefix='Fr.')
_sign(DEFAULT, moneyed.CHW, prefix='CHW')
_sign(DEFAULT, moneyed.CLF, prefix='UF')
_sign(DEFAULT, moneyed.CLP, prefix='CLP$')
_sign(DEFAULT, moneyed.CNY, prefix='¥')
_sign(DEFAULT, moneyed.COP, prefix='COL$')
_sign(DEFAULT, moneyed.COU, prefix='COU')
_sign(DEFAULT, moneyed.CRC, prefix='₡')
_sign(DEFAULT, moneyed.CUC, prefix='CUC$')
_sign(DEFAULT, moneyed.CUP, prefix='$MN')
_sign(DEFAULT, moneyed.CVE, prefix='Esc')
_sign(DEFAULT, moneyed.CZK, suffix=' Kč')
_sign(DEFAULT, moneyed.DJF, prefix='D₣')
_sign(DEFAULT, moneyed.DKK, suffix=' Dkr')
_sign(DEFAULT, moneyed.DOP, prefix='RD$')
_sign(DEFAULT, moneyed.DZD, prefix='دج')
_sign(DEFAULT, moneyed.EGP, prefix='ج.م.')
_sign(DEFAULT, moneyed.ERN, prefix='Nfk')
_sign(DEFAULT, moneyed.ETB, prefix='Br')
_sign(DEFAULT, moneyed.EUR, suffix=' €')
_sign(DEFAULT, moneyed.FJD, prefix='FJ$')
_sign(DEFAULT, moneyed.FKP, prefix='FK£')
_sign(DEFAULT, moneyed.GBP, prefix='GB£')
_sign(DEFAULT, moneyed.GEL, prefix='ლ')
_sign(DEFAULT, moneyed.GHS, prefix='₵')
_sign(DEFAULT, moneyed.GIP, prefix='GIP£')
_sign(DEFAULT, moneyed.GMD, prefix='D')
_sign(DEFAULT, moneyed.GNF, prefix='G₣')
_sign(DEFAULT, moneyed.GTQ, prefix='Q')
_sign(DEFAULT, moneyed.GYD, prefix='G$')
_sign(DEFAULT, moneyed.HKD, prefix='HK$')
_sign(DEFAULT, moneyed.HNL, prefix='L')
_sign(DEFAULT, moneyed.HRK, suffix=' kn')
_sign(DEFAULT, moneyed.HTG, prefix='G')
_sign(DEFAULT, moneyed.HUF, prefix='Ft')
_sign(DEFAULT, moneyed.IDR, prefix='Rp')
_sign(DEFAULT, moneyed.ILS, prefix='₪')
_sign(DEFAULT, moneyed.IMP, prefix='IM£')
_sign(DEFAULT, moneyed.INR, prefix='₹')
_sign(DEFAULT, moneyed.IQD, prefix='ع.د')
_sign(DEFAULT, moneyed.IRR, prefix='ریال')
_sign(DEFAULT, moneyed.ISK, suffix=' Íkr')
_sign(DEFAULT, moneyed.JMD, prefix='J$')
_sign(DEFAULT, moneyed.JOD, prefix='JD')
_sign(DEFAULT, moneyed.JPY, prefix='¥')
_sign(DEFAULT, moneyed.KES, prefix='Ksh')
_sign(DEFAULT, moneyed.KGS, prefix='лв')
_sign(DEFAULT, moneyed.KHR, prefix='៛')
_sign(DEFAULT, moneyed.KMF, prefix='C₣')
_sign(DEFAULT, moneyed.KPW, prefix='₩')
_sign(DEFAULT, moneyed.KRW, prefix='₩')
_sign(DEFAULT, moneyed.KWD, prefix='د.ك')
_sign(DEFAULT, moneyed.KYD, prefix='CI$')
_sign(DEFAULT, moneyed.LAK, prefix='₭')
_sign(DEFAULT, moneyed.LBP, prefix='LL')
_sign(DEFAULT, moneyed.LKR, prefix='₨')
_sign(DEFAULT, moneyed.LRD, prefix='LD$')
_sign(DEFAULT, moneyed.LSL, prefix='M')
_sign(DEFAULT, moneyed.LTL, prefix='Lt')
_sign(DEFAULT, moneyed.LVL, prefix='Ls')
_sign(DEFAULT, moneyed.LYD, prefix='ل.د')
_sign(DEFAULT, moneyed.MAD, prefix='د.م.')
_sign(DEFAULT, moneyed.MGA, prefix='Ar')
_sign(DEFAULT, moneyed.MKD, prefix='ден')
_sign(DEFAULT, moneyed.MMK, prefix='K')
_sign(DEFAULT, moneyed.MNT, prefix='₮')
_sign(DEFAULT, moneyed.MOP, prefix='MOP$')
_sign(DEFAULT, moneyed.MRO, prefix='UM')
_sign(DEFAULT, moneyed.MUR, prefix='₨')
_sign(DEFAULT, moneyed.MVR, prefix='Rf.')
_sign(DEFAULT, moneyed.MWK, prefix='MK')
_sign(DEFAULT, moneyed.MXN, prefix='Mex$')
_sign(DEFAULT, moneyed.MXV, prefix='UDI')
_sign(DEFAULT, moneyed.MYR, prefix='RM')
_sign(DEFAULT, moneyed.MZN, prefix='MT')
_sign(DEFAULT, moneyed.NAD, prefix='N$')
_sign(DEFAULT, moneyed.NGN, prefix='₦')
_sign(DEFAULT, moneyed.NIO, prefix='C$')
_sign(DEFAULT, moneyed.NOK, suffix=' Nkr')
_sign(DEFAULT, moneyed.NPR, prefix='₨')
_sign(DEFAULT, moneyed.NZD, prefix='NZ$')
_sign(DEFAULT, moneyed.OMR, prefix='ر.ع.')
_sign(DEFAULT, moneyed.PAB, prefix='B/.')
_sign(DEFAULT, moneyed.PEN, prefix='S/.')
_sign(DEFAULT, moneyed.PGK, prefix='K')
_sign(DEFAULT, moneyed.PHP, prefix='₱')
_sign(DEFAULT, moneyed.PKR, prefix='₨')
_sign(DEFAULT, moneyed.PLN, suffix=' zł')
_sign(DEFAULT, moneyed.PYG, prefix='₲')
_sign(DEFAULT, moneyed.QAR, prefix='ر.ق')
_sign(DEFAULT, moneyed.RSD, prefix='RSD ')
_sign(DEFAULT, moneyed.RUB, suffix=' руб.')
_sign(DEFAULT, moneyed.RWF, prefix='FRw')
_sign(DEFAULT, moneyed.SAR, prefix='ر.س')
_sign(DEFAULT, moneyed.SBD, prefix='SI$')
_sign(DEFAULT, moneyed.SCR, prefix='SRe')
_sign(DEFAULT, moneyed.SDG, prefix='S£')
_sign(DEFAULT, moneyed.SEK, suffix=' Skr')
_sign(DEFAULT, moneyed.SGD, prefix='S$')
_sign(DEFAULT, moneyed.SHP, prefix='SH£')
_sign(DEFAULT, moneyed.SLL, prefix='Le')
_sign(DEFAULT, moneyed.SOS, prefix='Sh.So.')
_sign(DEFAULT, moneyed.SRD, prefix='SRD$')
_sign(DEFAULT, moneyed.SSP, prefix='£')
_sign(DEFAULT, moneyed.STD, prefix='Db')
_sign(DEFAULT, moneyed.SVC, prefix='₡')
_sign(DEFAULT, moneyed.SYP, prefix='£S')
_sign(DEFAULT, moneyed.SZL, prefix='E')
_sign(DEFAULT, moneyed.THB, prefix='฿')
_sign(DEFAULT, moneyed.TND, prefix='د.ت')
_sign(DEFAULT, moneyed.TMT, prefix='m')
_sign(DEFAULT, moneyed.TOP, prefix='TOP$')
_sign(DEFAULT, moneyed.TRY, prefix='₺')
_sign(DEFAULT, moneyed.TTD, prefix='TT$')
_sign(DEFAULT, moneyed.TVD, prefix='$T')
_sign(DEFAULT, moneyed.TWD, prefix='NT$')
_sign(DEFAULT, moneyed.UAH, prefix='₴')
_sign(DEFAULT, moneyed.UGX, prefix='USh')
_sign(DEFAULT, moneyed.USD, prefix='US$')
_sign(DEFAULT, moneyed.USN, prefix='USN')
_sign(DEFAULT, moneyed.UYI, prefix='$U')
_sign(DEFAULT, moneyed.UYU, prefix='$U')
_sign(DEFAULT, moneyed.VEF, prefix='Bs.')
_sign(DEFAULT, moneyed.VND, prefix='₫')
_sign(DEFAULT, moneyed.VUV, prefix='VT')
_sign(DEFAULT, moneyed.WST, prefix='WS$')
_sign(DEFAULT, moneyed.XAF, prefix='FCFA')
_sign(DEFAULT, moneyed.XCD, prefix='EC$')
_sign(DEFAULT, moneyed.XDR, prefix='SDR')
_sign(DEFAULT, moneyed.XOF, prefix='CFA')
_sign(DEFAULT, moneyed.XSU, prefix='Sucre')
_sign(DEFAULT, moneyed.XUA, prefix='XUA')
_sign(DEFAULT, moneyed.XXX, prefix='XXX')
_sign(DEFAULT, moneyed.ZAR, prefix='R')
_sign(DEFAULT, moneyed.ZMK, prefix='ZK')
_sign(DEFAULT, moneyed.ZMW, prefix='ZK')
_sign(DEFAULT, moneyed.ZWL, prefix='Z$')

_sign('en_US', moneyed.USD, prefix='$')
_sign('en_GB', moneyed.GBP, prefix='£')
_sign('en_GB', moneyed.EUR, prefix='€')
_sign('sv_SE', moneyed.SEK, prefix='kr ')
_sign('pl_PL', moneyed.PLN, suffix=' zł')
_sign('de_DE', moneyed.EUR, suffix=' €')
_sign('de_AT', moneyed.EUR, suffix=' €')
_sign('de_CH', moneyed.CHF, prefix='Fr.')
_sign('fr_FR', moneyed.EUR, suffix=' €')
_sign('fr_FR', moneyed.USD, suffix=' $ US')
_sign('fr_CA', moneyed.USD, suffix=' $ US')
_sign('fr_FR', moneyed.CAD, suffix=' $ CA')
_sign('fr_CA', moneyed.CAD, suffix=' $')
_sign('fr_CA', moneyed.EUR, suffix=' €')
_sign('nl_NL', moneyed.EUR, prefix='€ ')
_sign('nb_NO', moneyed.NOK, prefix='kr ')
_sign('nn_NO', moneyed.NOK, prefix='kr ')

# Adding translations for missing currencies
_sign('en_US', moneyed.KWD, prefix='KD')
_sign('en_US', moneyed.BHD, prefix='BD')
_sign('en_US', moneyed.SAR, prefix='SR')
_sign('en_US', moneyed.DZD, prefix='DA')
_sign('en_US', moneyed.LYD, prefix='LD')
_sign('en_US', moneyed.TND, prefix='DT')
_sign('en_US', moneyed.AED, prefix='Dhs')
_sign('en_US', moneyed.EGP, prefix='L.E.')
_sign('en_US', moneyed.QAR, prefix='QR')



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, unicode_literals

import sys
import warnings
from decimal import Decimal

from babel import Locale
from babel.core import get_global

from .l10n import format_money
from .utils import cached_property

# Default, non-existent, currency
DEFAULT_CURRENCY_CODE = 'XYZ'

PYTHON2 = sys.version_info[0] == 2


def force_decimal(amount):
    """Given an amount of unknown type, type cast it to be a Decimal."""
    if not isinstance(amount, Decimal):
        return Decimal(str(amount))
    return amount


class Currency(object):
    """
    A Currency represents a form of money issued by governments, and
    used in one or more states/countries.  A Currency instance
    encapsulates the related data of: the ISO currency/numeric code, a
    canonical name, and countries the currency is used in.
    """

    def __init__(self, code='', numeric=None, sub_unit=1, name=None, countries=None):
        self.code = code
        self.numeric = numeric
        self.sub_unit = sub_unit
        if name is not None:
            self.name = name
        if countries is not None:
            self.countries = countries

    def __hash__(self):
        return hash(self.code)

    def __eq__(self, other):
        return type(self) is type(other) and self.code == other.code

    def __ne__(self, other):
        return not self.__eq__(other)

    def __repr__(self):
        return self.code

    def __lt__(self, other):
        return self.code < other.code

    def __le__(self, other):
        return self.code <= other.code

    @cached_property
    def name(self):
        """"
        Name of the currency in US locale. For backwards compat.

        Consider using get_name() instead, or babel.numbers.get_currency_name()
        """
        return self.get_name('en_US')

    def get_name(self, locale, count=None):
        from babel.numbers import get_currency_name
        return get_currency_name(self.code, locale=locale, count=count)

    @cached_property
    def countries(self):
        """
        List of country names, uppercased and in US locale, where the currency is
        used at present.

        DEPRECATED. Use `.country_codes` instead, and get_country_name() to
        convert these to a country name in your desired locale.

        """
        return [get_country_name(country_code, 'en_US').upper()
                for country_code in self.country_codes]

    @cached_property
    def country_codes(self):
        """
        List of current country codes for the currency.
        """
        return [
            territory.upper()
            for territory, currencies in get_global('territory_currencies').items()
            for currency_code, start, end, is_tender in currencies
            if end is None and currency_code == self.code
        ]


def get_country_name(country_code, locale):
    return Locale.parse(locale).territories[country_code]


class MoneyComparisonError(TypeError):
    # This exception was needed often enough to merit its own
    # Exception class.

    def __init__(self, other):
        assert not isinstance(other, Money)
        self.other = other

    def __str__(self):
        # Note: at least w/ Python 2.x, use __str__, not __unicode__.
        return "Cannot compare instances of Money and %s" \
               % self.other.__class__.__name__


class CurrencyDoesNotExist(Exception):

    def __init__(self, code):
        super(CurrencyDoesNotExist, self).__init__(
            "No currency with code %s is defined." % code)


class Money(object):
    """
    A Money instance is a combination of data - an amount and a
    currency - along with operators that handle the semantics of money
    operations in a better way than just dealing with raw Decimal or
    ($DEITY forbid) floats.
    """

    def __init__(self, amount=Decimal('0.0'), currency=DEFAULT_CURRENCY_CODE):
        if not isinstance(amount, Decimal):
            amount = Decimal(str(amount))
        self.amount = amount

        if not isinstance(currency, Currency):
            currency = get_currency(str(currency).upper())
        self.currency = currency

    def __repr__(self):
        return "Money('%s', '%s')" % (self.amount, self.currency)

    def __unicode__(self):
        return format_money(self)

    if PYTHON2:
        def __str__(self):
            # On Python 2, `__str__` returns byte strings, so we can't include unicode symbols.
            # Use a simpler fallback that avoids format_money
            return '{0}{1:,}'.format(self.currency.code, self.amount)

    else:
        def __str__(self):
            return format_money(self)

    def __hash__(self):
        return hash((self.amount, self.currency))

    def __pos__(self):
        return self.__class__(
            amount=self.amount,
            currency=self.currency)

    def __neg__(self):
        return self.__class__(
            amount=-self.amount,
            currency=self.currency)

    def __add__(self, other):
        if other == 0:
            # This allows things like 'sum' to work on list of Money instances,
            # just like list of Decimal.
            return self
        if not isinstance(other, Money):
            raise TypeError('Cannot add or subtract a ' +
                            'Money and non-Money instance.')
        if self.currency == other.currency:
            return self.__class__(
                amount=self.amount + other.amount,
                currency=self.currency)

        raise TypeError('Cannot add or subtract two Money ' +
                        'instances with different currencies.')

    def __sub__(self, other):
        return self.__add__(-other)

    def __rsub__(self, other):
        return (-self).__add__(other)

    def __mul__(self, other):
        if isinstance(other, Money):
            raise TypeError('Cannot multiply two Money instances.')
        else:
            if isinstance(other, float):
                warnings.warn("Multiplying Money instances with floats is deprecated", DeprecationWarning)
            return self.__class__(
                amount=(self.amount * force_decimal(other)),
                currency=self.currency)

    def __truediv__(self, other):
        if isinstance(other, Money):
            if self.currency != other.currency:
                raise TypeError('Cannot divide two different currencies.')
            return self.amount / other.amount
        else:
            if isinstance(other, float):
                warnings.warn("Dividing Money instances by floats is deprecated", DeprecationWarning)
            return self.__class__(
                amount=(self.amount / force_decimal(other)),
                currency=self.currency)

    def __rtruediv__(self, other):
        raise TypeError('Cannot divide non-Money by a Money instance.')

    def round(self, ndigits=0):
        """
        Rounds the amount using the current ``Decimal`` rounding algorithm.
        """
        if ndigits is None:
            ndigits = 0
        return self.__class__(
            amount=self.amount.quantize(Decimal('1e' + str(-ndigits))),
            currency=self.currency)

    def __abs__(self):
        return self.__class__(
            amount=abs(self.amount),
            currency=self.currency)

    def __bool__(self):
        return bool(self.amount)

    if PYTHON2:
        __nonzero__ = __bool__

    def __rmod__(self, other):
        """
        Calculate percentage of an amount.  The left-hand side of the
        operator must be a numeric value.

        Example:
        >>> money = Money(200, 'USD')
        >>> 5 % money
        USD 10.00
        """
        if isinstance(other, Money):
            raise TypeError('Invalid __rmod__ operation')
        else:
            if isinstance(other, float):
                warnings.warn("Calculating percentages of Money instances using floats is deprecated",
                              DeprecationWarning)
            return self.__class__(
                amount=(Decimal(str(other)) * self.amount / 100),
                currency=self.currency)

    __radd__ = __add__
    __rmul__ = __mul__

    # _______________________________________
    # Override comparison operators
    def __eq__(self, other):
        return (isinstance(other, Money) and
                (self.amount == other.amount) and
                (self.currency == other.currency))

    def __ne__(self, other):
        result = self.__eq__(other)
        return not result

    def __lt__(self, other):
        if not isinstance(other, Money):
            raise MoneyComparisonError(other)
        if (self.currency == other.currency):
            return (self.amount < other.amount)
        else:
            raise TypeError('Cannot compare Money with different currencies.')

    def __gt__(self, other):
        if not isinstance(other, Money):
            raise MoneyComparisonError(other)
        if (self.currency == other.currency):
            return (self.amount > other.amount)
        else:
            raise TypeError('Cannot compare Money with different currencies.')

    def __le__(self, other):
        return self < other or self == other

    def __ge__(self, other):
        return self > other or self == other

    def get_amount_in_sub_unit(self):
        return int(self.currency.sub_unit * self.amount)


# ____________________________________________________________________
# Definitions of ISO 4217 Currencies
# Source: http://www.iso.org/iso/support/faqs/faqs_widely_used_standards/widely_used_standards_other/currency_codes/currency_codes_list-1.htm  # noqa

CURRENCIES = {}
CURRENCIES_BY_ISO = {}


def add_currency(code, numeric, sub_unit=1, name=None, countries=None):
    currency = Currency(code=code, numeric=numeric, sub_unit=sub_unit, name=name, countries=countries)
    CURRENCIES[code] = currency
    # No lookup by numeric code for currencies without numeric codes
    if numeric is not None:
        CURRENCIES_BY_ISO[numeric] = currency
    return currency


def get_currency(code=None, iso=None):
    try:
        if iso:
            return CURRENCIES_BY_ISO[str(iso)]
        return CURRENCIES[code]
    except KeyError:
        raise CurrencyDoesNotExist(code)


def get_currencies_of_country(country_code):
    """
    Returns list with currency object(s) given the country's ISO-2 code.
    Raises a CountryDoesNotExist exception if the country is not found.

    country : str
    The full name of the country to be searched for.
    """
    country_code = country_code.upper()
    return sorted([
        currency
        for currency in CURRENCIES.values()
        if country_code in currency.country_codes
    ])


def list_all_currencies():
    return sorted(CURRENCIES.values(), key=lambda c: c.code)


DEFAULT_CURRENCY = add_currency(DEFAULT_CURRENCY_CODE, None, 1, 'Default currency.', [])

# The order of registration is important because we want active currencies to be available via `get_currency`.
# For this reason, we have obsolete currencies first, and then they may be overridden by active ones
# Obsolete currencies
ADP = add_currency('ADP', '020')
AFA = add_currency('AFA', '004', 100)
ALK = add_currency('ALK', '008')
AON = add_currency('AON', '024')
AOR = add_currency('AOR', '982')
ARA = add_currency('ARA', '032', 100)
ARP = add_currency('ARP', '032', 100)
ATS = add_currency('ATS', '040', 100)
AZM = add_currency('AZM', '031', 100)
BAD = add_currency('BAD', '070', 100)
BEF = add_currency('BEF', '056', 100)
BGL = add_currency('BGL', '100', 100)
BRC = add_currency('BRC', '076', 100)
BRE = add_currency('BRE', '076', 100)
BRN = add_currency('BRN', '076', 100)
BRR = add_currency('BRR', '987', 100)
BYR = add_currency('BYR', '974')
CLE = add_currency('CLE', '152')
CSD = add_currency('CSD', '891', 100)
CSK = add_currency('CSK', '200')
CYP = add_currency('CYP', '196', 100)
DDM = add_currency('DDM', '278')
DEM = add_currency('DEM', '276', 100)
ECS = add_currency('ECS', '218')
ECV = add_currency('ECV', '983', 100)
EEK = add_currency('EEK', '233', 100)
ESA = add_currency('ESA', '996')
ESB = add_currency('ESB', '995')
ESP = add_currency('ESP', '020')
FIM = add_currency('FIM', '246', 100)
FRF = add_currency('FRF', '250', 100)
GHC = add_currency('GHC', '288', 100)
GRD = add_currency('GRD', '300', 100)
GWP = add_currency('GWP', '624', 100)
HRD = add_currency('HRD', '191', 100)
IEP = add_currency('IEP', '372', 100)
ITL = add_currency('ITL', '380')
LTL = add_currency('LTL', '440', 100)
LUF = add_currency('LUF', '442', 100)
LVL = add_currency('LVL', '428', 100)
MGF = add_currency('MGF', '450')
MLF = add_currency('MLF', '466')
MRO = add_currency('MRO', '478', 100)
MTL = add_currency('MTL', '470', 100)
MZM = add_currency('MZM', '508', 100)
NLG = add_currency('NLG', '528', 100)
PEI = add_currency('PEI', '604')
PLZ = add_currency('PLZ', '616', 100)
PTE = add_currency('PTE', '620')
ROL = add_currency('ROL', '642')
RUR = add_currency('RUR', '810', 100)
SDD = add_currency('SDD', '736', 100)
SIT = add_currency('SIT', '705', 100)
SKK = add_currency('SKK', '703', 100)
SRG = add_currency('SRG', '740', 100)
STD = add_currency('STD', '678', 100)
TJR = add_currency('TJR', '762')
TMM = add_currency('TMM', '795', 100)
TPE = add_currency('TPE', '626')
TRL = add_currency('TRL', '792')
UAK = add_currency('UAK', '804', 100)
USS = add_currency('USS', '998', 100)
VEB = add_currency('VEB', '862', 100)
VEF = add_currency('VEF', '937', 100)
VNN = add_currency('VNN', '704')
XEU = add_currency('XEU', '954')
YDD = add_currency('YDD', '710')
YUM = add_currency('YUM', '891', 100)
YUN = add_currency('YUN', '890', 100)
ZAL = add_currency('ZAL', '991', 100)
ZMK = add_currency('ZMK', '894', 100)
ZRN = add_currency('ZRN', '180', 100)
ZRZ = add_currency('ZRZ', '180', 100)
ZWD = add_currency('ZWD', '716', 100)
ZWL = add_currency('ZWL', '932', 100)
ZWR = add_currency('ZWR', '935', 100)

# Further obsolete currencies that don't appear to have ISO 4217 codes
AOK = add_currency('AOK', None)
ARL = add_currency('ARL', None, 100)
ARM = add_currency('ARM', None, 100)
BAN = add_currency('BAN', None)
BEC = add_currency('BEC', None)
BEL = add_currency('BEL', None)
BGM = add_currency('BGM', None)
BGO = add_currency('BGO', None)
BOL = add_currency('BOL', None)
BOP = add_currency('BOP', None, 100)
BRB = add_currency('BRB', None, 100)
BRZ = add_currency('BRZ', None, 100)
BUK = add_currency('BUK', None)
BYB = add_currency('BYB', None, 100)
CNH = add_currency('CNH', None, 100)
CNX = add_currency('CNX', None, 100)
GEK = add_currency('GEK', None)
GNS = add_currency('GNS', None)
GQE = add_currency('GQE', None)
GWE = add_currency('GWE', None)
ILP = add_currency('ILP', None, 100)
ILR = add_currency('ILR', None, 100)
ISJ = add_currency('ISJ', None, 100)
KRH = add_currency('KRH', None)
KRO = add_currency('KRO', None)
LTT = add_currency('LTT', None, 100)
LUC = add_currency('LUC', None)
LUL = add_currency('LUL', None)
LVR = add_currency('LVR', None, 100)
MAF = add_currency('MAF', None, 100)
MCF = add_currency('MCF', None, 100)
MDC = add_currency('MDC', None)
MKN = add_currency('MKN', None)
MRU = add_currency('MRU', None, 100)
MTP = add_currency('MTP', None)
MVP = add_currency('MVP', None)
MXP = add_currency('MXP', None)
MZE = add_currency('MZE', None, 100)
NIC = add_currency('NIC', None, 100)
PES = add_currency('PES', None, 100)
RHD = add_currency('RHD', None, 100)
SDP = add_currency('SDP', None)
STN = add_currency('STN', None, 100)
SUR = add_currency('SUR', None)
UGS = add_currency('UGS', None)
UYP = add_currency('UYP', None, 100)
UYW = add_currency('UYW', None, 10000)
VES = add_currency('VES', None, 100)
XRE = add_currency('XRE', None)
YUD = add_currency('YUD', None, 100)
YUR = add_currency('YUR', None, 100)


# Active currencies
AED = add_currency('AED', '784', 100)
AFN = add_currency('AFN', '971', 100)
ALL = add_currency('ALL', '008', 100)
AMD = add_currency('AMD', '051', 100)
ANG = add_currency('ANG', '532', 100)
AOA = add_currency('AOA', '973', 100)
ARS = add_currency('ARS', '032', 100)
AUD = add_currency('AUD', '036', 100)
AWG = add_currency('AWG', '533', 100)
AZN = add_currency('AZN', '944', 100)
BAM = add_currency('BAM', '977', 100)
BBD = add_currency('BBD', '052', 100)
BDT = add_currency('BDT', '050', 100)
BGN = add_currency('BGN', '975', 100)
BHD = add_currency('BHD', '048', 1000)
BIF = add_currency('BIF', '108')
BMD = add_currency('BMD', '060', 100)
BND = add_currency('BND', '096', 100)
BOB = add_currency('BOB', '068', 100)
BOV = add_currency('BOV', '984', 100)
BRL = add_currency('BRL', '986', 100)
BSD = add_currency('BSD', '044', 100)
BTN = add_currency('BTN', '064', 100)
BWP = add_currency('BWP', '072', 100)
BYN = add_currency('BYN', '933', 100)
BZD = add_currency('BZD', '084', 100)
CAD = add_currency('CAD', '124', 100)
CDF = add_currency('CDF', '976', 100)
CHE = add_currency('CHE', '947', 100)
CHF = add_currency('CHF', '756', 100)
CHW = add_currency('CHW', '948', 100)
CLF = add_currency('CLF', '990', 10000)
CLP = add_currency('CLP', '152')
CNY = add_currency('CNY', '156', 100)
COP = add_currency('COP', '170', 100)
COU = add_currency('COU', '970', 100)
CRC = add_currency('CRC', '188', 100)
CUC = add_currency('CUC', '931', 100)
CUP = add_currency('CUP', '192', 100)
CVE = add_currency('CVE', '132', 100)
CZK = add_currency('CZK', '203', 100)
DJF = add_currency('DJF', '262')
DKK = add_currency('DKK', '208', 100)
DOP = add_currency('DOP', '214', 100)
DZD = add_currency('DZD', '012', 100)
EGP = add_currency('EGP', '818', 100)
ERN = add_currency('ERN', '232', 100)
ETB = add_currency('ETB', '230', 100)
EUR = add_currency('EUR', '978', 100)
FJD = add_currency('FJD', '242', 100)
FKP = add_currency('FKP', '238', 100)
GBP = add_currency('GBP', '826', 100)
GEL = add_currency('GEL', '981', 100)
GHS = add_currency('GHS', '936', 100)
GIP = add_currency('GIP', '292', 100)
GMD = add_currency('GMD', '270', 100)
GNF = add_currency('GNF', '324')
GTQ = add_currency('GTQ', '320', 100)
GYD = add_currency('GYD', '328', 100)
HKD = add_currency('HKD', '344', 100)
HNL = add_currency('HNL', '340', 100)
HRK = add_currency('HRK', '191', 100)
HTG = add_currency('HTG', '332', 100)
HUF = add_currency('HUF', '348', 100)
IDR = add_currency('IDR', '360', 100)
ILS = add_currency('ILS', '376', 100)
IMP = add_currency('IMP', None, 100)
INR = add_currency('INR', '356', 100)
IQD = add_currency('IQD', '368', 1000)
IRR = add_currency('IRR', '364', 100)
ISK = add_currency('ISK', '352')
JMD = add_currency('JMD', '388', 100)
JOD = add_currency('JOD', '400', 1000)
JPY = add_currency('JPY', '392')
KES = add_currency('KES', '404', 100)
KGS = add_currency('KGS', '417', 100)
KHR = add_currency('KHR', '116', 100)
KMF = add_currency('KMF', '174')
KPW = add_currency('KPW', '408', 100)
KRW = add_currency('KRW', '410')
KWD = add_currency('KWD', '414', 1000)
KYD = add_currency('KYD', '136', 100)
KZT = add_currency('KZT', '398', 100)
LAK = add_currency('LAK', '418', 100)
LBP = add_currency('LBP', '422', 100)
LKR = add_currency('LKR', '144', 100)
LRD = add_currency('LRD', '430', 100)
LSL = add_currency('LSL', '426', 100)
LYD = add_currency('LYD', '434', 1000)
MAD = add_currency('MAD', '504', 100)
MDL = add_currency('MDL', '498', 100)
MGA = add_currency('MGA', '969', 100)
MKD = add_currency('MKD', '807', 100)
MMK = add_currency('MMK', '104', 100)
MNT = add_currency('MNT', '496', 100)
MOP = add_currency('MOP', '446', 100)
MUR = add_currency('MUR', '480', 100)
MVR = add_currency('MVR', '462', 100)
MWK = add_currency('MWK', '454', 100)
MXN = add_currency('MXN', '484', 100)
MXV = add_currency('MXV', '979', 100)
MYR = add_currency('MYR', '458', 100)
MZN = add_currency('MZN', '943', 100)
NAD = add_currency('NAD', '516', 100)
NGN = add_currency('NGN', '566', 100)
NIO = add_currency('NIO', '558', 100)
NOK = add_currency('NOK', '578', 100)
NPR = add_currency('NPR', '524', 100)
NZD = add_currency('NZD', '554', 100)
OMR = add_currency('OMR', '512', 1000)
PAB = add_currency('PAB', '590', 100)
PEN = add_currency('PEN', '604', 100)
PGK = add_currency('PGK', '598', 100)
PHP = add_currency('PHP', '608', 100)
PKR = add_currency('PKR', '586', 100)
PLN = add_currency('PLN', '985', 100)
PYG = add_currency('PYG', '600')
QAR = add_currency('QAR', '634', 100)
RON = add_currency('RON', '946', 100)
RSD = add_currency('RSD', '941', 100)
RUB = add_currency('RUB', '643', 100)
RWF = add_currency('RWF', '646')
SAR = add_currency('SAR', '682', 100)
SBD = add_currency('SBD', '090', 100)
SCR = add_currency('SCR', '690', 100)
SDG = add_currency('SDG', '938', 100)
SEK = add_currency('SEK', '752', 100)
SGD = add_currency('SGD', '702', 100)
SHP = add_currency('SHP', '654', 100)
SLL = add_currency('SLL', '694', 100)
SOS = add_currency('SOS', '706', 100)
SRD = add_currency('SRD', '968', 100)
SSP = add_currency('SSP', '728', 100)
SVC = add_currency('SVC', '222', 100)
SYP = add_currency('SYP', '760', 100)
SZL = add_currency('SZL', '748', 100)
THB = add_currency('THB', '764', 100)
TJS = add_currency('TJS', '972', 100)
TMT = add_currency('TMT', '934', 100)
TND = add_currency('TND', '788', 1000)
TOP = add_currency('TOP', '776', 100)
TRY = add_currency('TRY', '949', 100)
TTD = add_currency('TTD', '780', 100)
TVD = add_currency('TVD', None, 100)
TWD = add_currency('TWD', '901', 100)
TZS = add_currency('TZS', '834', 100)
UAH = add_currency('UAH', '980', 100)
UGX = add_currency('UGX', '800')
USD = add_currency('USD', '840', 100)
USN = add_currency('USN', '997', 100)
UYI = add_currency('UYI', '940')
UYU = add_currency('UYU', '858', 100)
UZS = add_currency('UZS', '860', 100)
VND = add_currency('VND', '704')
VUV = add_currency('VUV', '548')
WST = add_currency('WST', '882', 100)
XAF = add_currency('XAF', '950')
XAG = add_currency('XAG', '961')
XAU = add_currency('XAU', '959')
XBA = add_currency('XBA', '955')
XBB = add_currency('XBB', '956')
XBC = add_currency('XBC', '957')
XBD = add_currency('XBD', '958')
XCD = add_currency('XCD', '951', 100)
XDR = add_currency('XDR', '960')
XFO = add_currency('XFO', None)
XFU = add_currency('XFU', None)
XOF = add_currency('XOF', '952')
XPD = add_currency('XPD', '964')
XPF = add_currency('XPF', '953')
XPT = add_currency('XPT', '962')
XSU = add_currency('XSU', '994')
XTS = add_currency('XTS', '963')
XUA = add_currency('XUA', '965')
XXX = add_currency(
    'XXX',
    '999',
    # For backwards compat we keep values here, instead of getting
    # Babel's data.
    name='The codes assigned for transactions where no currency is involved',
    countries=['ZZ07_No_Currency'],
)
YER = add_currency('YER', '886', 100)
ZAR = add_currency('ZAR', '710', 100)
ZMW = add_currency('ZMW', '967', 100)
ZWN = add_currency('ZWN', '942', 100)



#!/usr/bin/env python

import sys

from setuptools import setup
from setuptools.command.test import test as TestCommand


class Tox(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        # import here, cause outside the eggs aren't loaded
        import tox
        errno = tox.cmdline(self.test_args)
        sys.exit(errno)


setup(
    name='py-moneyed',
    packages=['moneyed'],
    version='1.1',
    description='Provides Currency and Money classes for use in your Python code.',
    author='Kai',
    author_email='k@limist.com',
    url='http://github.com/py-moneyed/py-moneyed',
    download_url='',
    keywords="money currency class abstraction",
    license='BSD',
    install_requires=[
        'babel>=2.8.0',
    ],
    classifiers=[
        'Programming Language :: Python',
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Development Status :: 6 - Mature',
        'Environment :: Other Environment',
        'Intended Audience :: Developers',
        'Topic :: Office/Business :: Financial',
        'Topic :: Software Development :: Libraries :: Python Modules'],
    long_description=open('README.rst', 'r').read(),
    long_description_content_type="text/x-rst",
    extras_require={
        'tests': [
            'pytest>=2.3.0',
            'tox>=1.6.0'
        ]},
    tests_require=['tox>=1.6.0', 'pytest>=2.3.0'],
    cmdclass={'test': Tox},
    include_package_data=True,
)
